@@ -364,16 +364,22 @@
 //! </table>
 
 #![no_std]
-#![doc(html_root_url = "https://docs.rs/cxx/1.0.94")]
+#![doc(html_root_url = "https://docs.rs/cxx/1.0.111")]
+#![cfg_attr(doc_cfg, feature(doc_cfg))]
 #![deny(
     improper_ctypes,
     improper_ctypes_definitions,
     missing_docs,
     unsafe_op_in_unsafe_fn
 )]
-#![cfg_attr(doc_cfg, feature(doc_cfg))]
+#![warn(
+    clippy::alloc_instead_of_core,
+    clippy::std_instead_of_alloc,
+    clippy::std_instead_of_core
+)]
 #![allow(non_camel_case_types)]
 #![allow(
+    clippy::cast_possible_truncation,
     clippy::cognitive_complexity,
     clippy::declare_interior_mutable_const,
     clippy::doc_markdown,
@@ -392,8 +398,10 @@
     clippy::new_without_default,
     clippy::or_fun_call,
     clippy::ptr_arg,
+    clippy::ptr_as_ptr,
     clippy::toplevel_ref_arg,
     clippy::transmute_undefined_repr, // clippy bug: https://github.com/rust-lang/rust-clippy/issues/8417
+    clippy::uninlined_format_args,
     clippy::useless_let_if_seq,
     clippy::wrong_self_convention
 )]
